// 解析器管理
import { Component, SetupContext } from "vue";
import { SimpleManager, Cache } from "./common/simple";
import { NormalizedSchemaNode, ComponentNode } from "../schema";
import { PluginTypes } from "../constant";
import { ParserUtil } from "../components/renderer";

export type UIName = string

export type UIConfig = {
    [key: string]: Component,
}

export type UIOptions = {
    name: UIName, // 组件名称
    component: Component, // 组件内容
}

// 组件解析插件
export type ParserPlugin = {
    name: string,  // 插件名称
    type: PluginTypes.Parser, // 插件类型
    reload?: string, // 要重载的插件的名称，没有定义此项时，有重名的Plugin报错
    scopedReload?: string, // 用于局部重载，重载范围内的子类型解析器名称
    reloadScope?: string, // 用于局部重载，重载范围的父解析器名称
    ui?: UIName | UIOptions,  // ui组件相关配置
    parse(options: NormalizedSchemaNode): ComponentNode, // 解析逻辑，每次渲染都会被调用
    parse(options: NormalizedSchemaNode, context: SetupContext): ComponentNode, // 重载解析逻辑
    parse(options: NormalizedSchemaNode, context: SetupContext, util: ParserUtil): ComponentNode, // 重载解析逻辑
    created?: () => void,
    [key: string]: any,
}

type ParserCache = Cache<ParserPlugin>

export class ParserManager extends SimpleManager{
    constructor(){
        super()
        this.name = 'Parser'
        this.carReset = false
    }
}