/**
 * 通过解析器
 */
import { SetupContext, Ref, ref } from 'vue'
import { ComponentNode, SchemaNodeList, NormalizedSchemaNode } from '../schema'
import { ParserPlugin } from '../manager/parser'
import { Util } from '../plugins'
import { toPascalCase } from '../util'
import { UITypes, ParserTypes, PluginTypes } from '../constant'

export default {
    name: ParserTypes.Button,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Button,
    parse: ({
        label,
        type,
        meta,
    }: NormalizedSchemaNode, context: SetupContext, {
        pluginDriver
    }: Util) => {
        const component = pluginDriver.getUI(toPascalCase(type))
        const children: Ref<SchemaNodeList<string>> = ref([])
        const onClick = () => pluginDriver.handleAction(meta)

        if(label) children.value.push(label)

        return {
            component,
            props: {
            onClick,
            },
            children
        } as ComponentNode
    },
} as ParserPlugin