/**
 * 通过解析器
 */
import { toRef, reactive, SetupContext } from 'vue'
import { ComponentNode, Data, FormItemSchema, NormalizedSchemaNode } from '../../schema'
import { ParserPlugin } from '../../manager/parser'
import { ParserTypes, PluginTypes, UITypes } from '../../constant'
import { Util } from '../../plugins'
import { mergeParams } from '../../util/http'

type OptionMeta = {
    label: string
    value: string | number | boolean
    [key: string]: any
}

export type FormItemMeta = {
    items?: Array<OptionMeta>
}

export type FormMeta = {
    filter: string // 提交时筛选table，对应table的key
}

const getDefaultValue = (item: FormItemSchema): any => {
    if(item.type === 'checkbox'){
        return []
    }else{
        return ''
    }
}

export default {
    name: ParserTypes.Form,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Form,
    parse: (schema: NormalizedSchemaNode, context: SetupContext, {
        pluginDriver
    }: Util) => {
        const {meta} = schema
        const items = toRef(schema, 'items')
        const model: Data = reactive({})
        const onUpdateItemValue = (key: string, value: any) => {
            model[key] = value
        }
        const onSubmit = (model: object) => {
            mergeParams(meta, model)
            pluginDriver.handleAction(meta, model)
        }

        items.value?.forEach( item => {
            model[item.key] = item.value || getDefaultValue(item as FormItemSchema)
        })

        return {
            props: {
                ...meta,
                items,
                model,
                onSubmit,
            },
            events: {
                onUpdateItemValue
            }
        } as ComponentNode
    },
} as ParserPlugin