import { ref } from 'vue'
import { SchemaRenderer } from '../../components/renderer'
import { AppConfig } from '../../engine'
import { http } from '../../util/http'
import { useRouter, useRoute, RouteRecordRaw } from 'vue-router'
import { ParserTypes } from '../../constant'

type MenuItem = {
    index?: boolean // 是否为首页
    mkey: string,
    label: string, // 菜单文案
    key: string, // 路由路径及name
    subs?: MenuItem[]
}

// kcp menus接口返回的数据
type MenusResponse = {
    appTitle: string
    menus: Array<MenuItem>
    subPages: Array<MenusResponse>
}

// 将接口返回的菜单配置，转换为routes
const menusToRoutes = (menus: Array<MenuItem>, level: number = 1): any => menus.map(({
    key,
    label: title,
    subs,
    index,
}) => ({
    name: key,
    path: level > 1 ? key: `/${key}`,
    meta: {
        title,
        index,
    },
    component: SchemaRenderer,
    props: {
        uniqueKey: key,
        type: ParserTypes.Page
    },
    children: subs && menusToRoutes(subs, level + 1)
}))

export default function useMenus(appConfig: AppConfig){
    const isInit = ref(false)
    const routes = ref([])
    const title = ref(appConfig?.title)
    const { kcpApi, menuApiPath, appKey } = appConfig
    const router = useRouter()
    const currentRoute = useRoute()
    console.log('currentRoute', currentRoute.name, currentRoute.path);

    const getRoutes = async () => {
        const res: MenusResponse = await http({
            url: kcpApi + menuApiPath,
            method: 'get',
            params: {
                app_key: appKey
            }
        })

        if(!res) return

        const { appTitle, menus } = res
        const newRoutes = menusToRoutes(menus)

        newRoutes.forEach((route: RouteRecordRaw) => router.addRoute(route));

        title.value = appTitle
        routes.value = newRoutes

        isInit.value = true
    }

    getRoutes().then( () => {
        // 路由正确解析，直接返回
        if(currentRoute.name) return

        const routes = router.getRoutes()
        const indexRoute = routes.find( route => route.meta.index )
        const routeOfCurrentPath = router.resolve(currentRoute.path)
        console.log('resolved router', router.resolve(currentRoute.path), indexRoute);

        if(routeOfCurrentPath.name){
            router.push(routeOfCurrentPath)
        }else if(indexRoute){
            router.push(indexRoute)
        }
    })

    return {
        isInit,
        title,
        currentRoute,
        routes,
        getRoutes
    }
}