/**
 * 建站平台框架，对应解析器
 */
import { ref, SetupContext, watch } from 'vue'
import { SchemaNode, ComponentNode, SchemaNodeList, NormalizedSchemaNode, TreeSchema } from '../schema'
import { PluginUtil } from '../plugins'
import { ParserPlugin } from '../manager/parser'
import { ParserTypes, PluginTypes, UITypes } from '../constant'
import { AppConfig } from '../engine'

 type PageResponse = {
     pageJSON?: SchemaNode[]
 }

 const DEFAULT_PROPS = {
    direction: 'vertical'
 }

export default {
    name: ParserTypes.Page,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Stack,
    parse: (props: NormalizedSchemaNode, context: SetupContext, {
        appConfig,
        http,
    }: PluginUtil) => {
        const isInit = ref(false)
        const {kcpApi, pageApiPath} = appConfig as AppConfig
        const children = ref(props.items)
        const getPageJSON = async () => {
            const res = await http({
                url: kcpApi + pageApiPath,
                method: 'get',
                params: {
                    pageKey: props.uniqueKey
                }
            })

            if(!res) return

            children.value = (res as PageResponse)?.pageJSON as SchemaNodeList

            isInit.value = true
        }

        getPageJSON()

        watch( () => props.uniqueKey, () => {
            isInit.value = false
            getPageJSON()
        })

        return {
            isInit,
            props: {
                ...DEFAULT_PROPS,
                ...props.meta,
            },
            children
        } as ComponentNode
    },
} as ParserPlugin