/**
 * 通过解析器
 */
import { toRef, ref, SetupContext, Ref, reactive, computed } from 'vue'
import { NormalizedSchemaNode, ComponentNode, Meta, AllMeta, Data } from '../../schema'
import { ParserPlugin } from '../../manager/parser'
import { ParserTypes, PluginTypes, UITypes } from '../../constant'
import { Util } from '../../plugins'
import useActions from './useActions'
import { PageChangeTypes } from './usePagination'
import useRequest from './useRequest'

export type TableMethods = {
    refresh(isInit?: boolean): Promise<void>,
    setExtendParams(newV: Data): void
}

export default {
    name: ParserTypes.Table,  // 插件名称
    type: PluginTypes.Parser,
    ui: UITypes.Table,
    parse: (schema: NormalizedSchemaNode, context: SetupContext, util: Util) => {
        const isInit = ref(false)
        const meta: Ref<Meta> = toRef(schema, 'meta')

        const { pagination, refresh, data, handlePageChange, setExtendParams } = useRequest(meta, util)
        const { columns } = useActions(schema, util.pluginDriver)

        refresh(true).then(() => {
            isInit.value = true
        })

        return {
            isInit,
            props: {
                columns,
                data,
                pagination,
                'onUpdate:currentPage': (value: number) => handlePageChange(PageChangeTypes.CurrentPage, value),
                'onUpdate:pageSize': (value: number) => handlePageChange(PageChangeTypes.PageSize, value)
            },
            methods: {
                refresh,
                setExtendParams
            } as TableMethods
        } as ComponentNode
    },
} as ParserPlugin