import { Component, Ref } from 'vue'
import { ActionMeta } from '../manager/action'
import { TableMeta } from '../parsers/table/usePagination'
import { FormItemMeta } from '../parsers/form'

export type AllMeta = ActionMeta & TableMeta & FormItemMeta & Record<string, any>

export type Meta = AllMeta | undefined

export interface BaseSchema {
    key: string
    type: string
    label?: string
    meta?: Meta
}

export interface TreeSchema<T = SchemaNode> extends BaseSchema{
    items?: SchemaNodeList<T>
}

type FormItemReg = {
    pattern: string
    message: string
}

export interface FormItemSchema extends BaseSchema{
    value?: any
    required: boolean
    reg: FormItemReg
}

export interface ActionSchema extends BaseSchema{
    onClick?(): void
}

export interface TableActionsSchema extends BaseSchema{
    actions?: SchemaNodeList<ActionSchema>
}

export type SchemaNode = FormItemSchema & ActionSchema & TableActionsSchema & TreeSchema

export type NormalizedSchema = {
    uniqueKey: string
    key?: string
}

export type NormalizedSchemaNode<T = SchemaNode> = T | NormalizedSchema

export type SchemaNodeList<T = SchemaNode> = Array<T>

// ActionMeta
type Action = {
    [key: string]: any
}

export type ComponentNode = {
    component?: Component, // 组件,
    isInit?: Ref<boolean>,  // 组件是否初始化完成，需要异步初始化的组件使用
    data?: Record<string, any>, // 组件的data属性，将被添加到属性中
    props?: Record<string, any>, // 组件属性
    events?: Record<string, Action>,  // 组件事件{ change: {url: ''} }
    methods?: Record<string, Function>,
    ref?: string,
    slot?: string,
    children?: Ref<SchemaNodeList<SchemaNode | string>>, // parser只对本组件相关进行解析，children不做转换，递归调用交给ParserComponent去解析
}

export type ComponentNodeList = Array<ComponentNode | string>

type Page = Component

// 组件树通用描述
type ComponentTree = {
    component: Page, //组件
    props: Record<string, any>, // 组件属性
    events: Record<string, Action>,  // 组件事件{ change: {url: ''} }
    ref: string,
    slot: string,
    children: Ref<ComponentNodeList>,
}

export type Data = Record<string, any>

export type RestArguments = Array<any>