import Axios, { AxiosInstance, AxiosRequestConfig } from 'axios'
import { Data, Meta } from '../schema'
import { pluginDriver } from '../plugins'
import { ActionMeta } from '../manager/action'
import { isObject, mergeObjects } from './index'
import { RequestMeta } from '../actions/request'
import { AppConfig } from '../engine'

const config = {
    timeout: 5000,
    withCredentials: true,
    // paramsSerializer(params: object) {
    //     return qs.stringify(params, { arrayFormat: 'brackets' });
    // },
}

const NEED_LOGIN_CODES = [
    401,
    201,
    40003,
    50003,
    201,
    401,
    40012,
    60000001,
    60000000,
    -911111
]

const needLogin = (code: number): boolean =>  NEED_LOGIN_CODES.includes(code)

const isSuccess = (code: number): boolean => code === 0

export type Http = AxiosInstance

export type Response = {
    code: number
    data?: any
    message: string
}

const axios = Axios.create(config)

axios.interceptors.response.use( response => {
    const { data: responseData } = response
    const { code, data } = responseData as Response || {}

    if(needLogin(code)){
        const {location} = window
        const {loginUrl} = pluginDriver.getAppConfig() as AppConfig

        location.replace(`${loginUrl}?returnUrl=${encodeURIComponent(location.href)}`)
    }

    return isSuccess(code) ? data : Promise.reject(responseData)
})

export type HttpRequestConfig = AxiosRequestConfig

export function http(options: HttpRequestConfig): Promise<any>{
    return axios.request(options).then( res => {
        // TODO alias相关逻辑
        return res
    })
}

export const mergeParams = (meta: Meta, extendData: Data): Meta  => {
    if(!meta) return

    const { method, data, params } = meta as ActionMeta

    if((!method || method.toUpperCase() === 'POST') && isObject(data)){
        mergeObjects(data, extendData)
    }else if(isObject(params)){
        mergeObjects(params, extendData)
    }

    return meta
}