import { Data, SchemaNode, NormalizedSchemaNode, NormalizedSchema } from '../schema'
/**
 * 将字符串转为驼峰形式
 */
export const toCamelCase = (str: string): string => str.replace(/[-_](\w)/g, (matched, letter) => letter.toUpperCase())

/**
 * 将驼峰形式字符串转为大驼峰(帕斯卡)形式
 */
export const toPascalCase = (str: string): string => toCamelCase(str).replace(/^\w/, letter => letter.toUpperCase());

export const isObject = (val: unknown): val is Record<any, any> => val !== null && typeof val === 'object'

export const isFunction = (val: unknown): val is Function => typeof val === 'function'

export const isPromise = <T = any>(val: unknown): val is Promise<T> => {
    return isObject(val) && isFunction(val.then) && isFunction(val.catch)
}

export const isUndefined = (val: unknown): val is undefined => val === undefined

export const isFalse = (val: unknown): val is false => val === false

export const isString = (val: unknown): val is string => typeof val === 'string'

export const mergeObjects = (target: Data, source: Data): Data => {
    Object.entries(source).forEach(([key, value]) => {
        target[key] = value
    })

    return target
}

export const normalizeSchema = (schema: SchemaNode): NormalizedSchemaNode => {
    const normalizedSchema: NormalizedSchemaNode = Object.assign({
        uniqueKey: schema.key
    } as NormalizedSchema, schema)

    delete normalizedSchema.key

    return normalizedSchema
}